@@ -40,11 +40,26 @@
         status[status] <- file.access(ulibs[status], 2L) == 0
 
     status <- status[match(libs, ulibs)]
-    if (!all(status))
-        .message(
-            "Installation path not writeable, unable to update packages: %s",
-            paste(pkgs[!status, "Package"], collapse=", ")
+    if (!all(status)) {
+        failed_pkgs <- pkgs[!status, "Package"]
+        failed_lib <- pkgs[!status, "LibPath"]
+        failed <- split(failed_pkgs, failed_lib)
+        detail <- paste(
+            mapply(function(lib, pkg) {
+                paste0(
+                    "  path: ", lib, "\n",
+                    "  packages:\n",
+                    .msg(paste(pkg, collapse = ", "), indent = 4, exdent = 4)
+                )
+            }, names(failed), unname(failed), USE.NAMES = FALSE),
+            collapse = "\n"
         )
+        message(
+            .msg("Installation paths not writeable, unable to update packages"),
+            "\n",
+            detail
+        )
+    }
 
     pkgs[status,, drop=FALSE]
 }
@@ -55,6 +70,24 @@
     grep("^(https?://.*|[^/]+)$", pkgs, invert = invert, value=TRUE)
 }
 
+.install_filter_up_to_date <-
+    function(pkgs, instPkgs, old_pkgs, force)
+{
+    if (!force) {
+        noInst <- !pkgs %in% rownames(old_pkgs) & pkgs %in% rownames(instPkgs)
+        if (any(noInst))
+            .warning(
+                paste(
+                    "package(s) not installed when version(s) same",
+                    "as current; use `force = TRUE` to re-install: \n'%s'"
+                ),
+                paste(pkgs[noInst], collapse = "' '")
+            )
+        pkgs <- pkgs[!noInst]
+    }
+    pkgs
+}
+
 .install_filter_github_repos <-
     function(pkgs)
 {
@@ -72,7 +105,7 @@
             "%s\n    %s\n%s",
             "package 'remotes' not installed in library path(s)",
             paste(lib.loc, collapse="\n    "),
-            "install with 'install(\"remotes\")'",
+            "install with 'BiocManager::install(\"remotes\")'",
             call. = FALSE,
             wrap. = FALSE
         )
@@ -93,19 +126,23 @@
 }
 
 .install_repos <-
-    function(pkgs, lib, repos, ...)
+    function(pkgs, old_pkgs, instPkgs, lib, repos, force, ...)
 {
-    doing <- .install_filter_r_repos(pkgs)
+    doing <- .install_filter_up_to_date(
+        pkgs = pkgs, instPkgs = instPkgs, old_pkgs = old_pkgs, force = force
+    )
+    up_to_date <- setdiff(pkgs, doing)
+    doing <- .install_filter_r_repos(doing)
     if (length(doing)) {
         pkgNames <- paste(.sQuote(doing), collapse=", ")
         .message("Installing package(s) %s", pkgNames)
         .inet_install.packages(pkgs = doing, lib = lib, repos = repos, ...)
     }
-    setdiff(pkgs, doing)
+    setdiff(pkgs, c(doing, up_to_date))
 }
 
 .install_github <-
-    function(pkgs, lib, lib.loc, repos = repos, ...)
+    function(pkgs, lib, lib.loc, repos, force, ...)
 {
     doing <- .install_filter_github_repos(pkgs)
 
@@ -116,7 +153,7 @@
         .message("Installing github package(s) %s", pkgNames)
         .install_github_load_remotes(pkgs, lib.loc = lib.loc)
         for (repo in doing)
-            remotes::install_github(repo, lib = lib, ...)
+            remotes::install_github(repo, lib = lib, force = force, ...)
     }
     setdiff(pkgs, doing)
 }
@@ -151,20 +188,24 @@
 }
 
 .install <-
-    function(pkgs, repos, lib.loc=NULL, lib=.libPaths()[1], ...)
+    function(pkgs, old_pkgs, instPkgs, repos, lib.loc=NULL, lib=.libPaths()[1],
+        checkBuilt, force, ...)
 {
     requireNamespace("utils", quietly=TRUE) ||
         .stop("failed to load package 'utils'")
 
-    todo <- .install_repos(pkgs, lib = lib, repos = repos, ...)
+    todo <- .install_repos(
+        pkgs, old_pkgs, instPkgs = instPkgs, lib = lib, repos = repos,
+        checkBuilt = checkBuilt, force = force, ...
+    )
     todo <- .install_github(
-        todo, lib = lib, lib.loc = lib.loc, repos = repos, ...
+        todo, lib = lib, lib.loc = lib.loc, repos = repos, force = force, ...
     )
 
     if (length(todo))
         .warning(
-            "packages not installed (unknown repository)",
-            "\n    ", paste(.sQuote(todo), collapse = ", ")
+            "packages not installed (unknown repository)\n  '%s'",
+            paste(.sQuote(todo), collapse = "' '")
         )
 
     setdiff(pkgs, todo)
@@ -198,12 +239,12 @@
     }
 
     .inet_update.packages(
-        lib.loc, repos, oldPkgs = old_pkgs, ask = ask, instlib = instlib
+        lib.loc, repos, oldPkgs = old_pkgs, ask = ask, instlib = instlib, ...
     )
 }
 
 .install_updated_version <-
-    function(valid, update, repos, ...)
+    function(valid, update, old_pkgs, instPkgs, repos, ...)
 {
     if (isTRUE(valid))
         return(valid)
@@ -213,7 +254,7 @@
     if (is.null(pkgs) || !update)
         return(pkgs)
 
-    .install(pkgs, repos, ...)
+    .install(pkgs, old_pkgs, instPkgs, repos, ...)
     pkgs
 }
 
@@ -246,6 +287,9 @@
 #' and `remotes:::install()`. A typical use is to build vignettes, via
 #' `dependencies=TRUE, build_vignettes=TRUE`.
 #'
+#' See `?repositories` for additional detail on customizing where
+#' BiocManager searches for package installation.
+#'
 #' \env{BIOCONDUCTOR_ONLINE_VERSION_DIAGNOSIS} is an environment
 #' variable or global `options()` which, when set to `FALSE`, avoids
 #' the R and _Bioconductor_ version checks that are done by querying
@@ -253,7 +297,10 @@
 #' \env{BIOCONDUCTOR_ONLINE_VERSION_DIAGNOSIS} to `FALSE` can speed
 #' package loading when internet access is slow or non-existent, but
 #' may result in out-of-date information about the current release and
-#' development versions of _Bioconductor_.
+#' development versions of _Bioconductor_. Offline users should
+#' set the \env{BIOCONDUCTOR_CONFIG_FILE} environment variable or option
+#' to a `.yaml` file similar to \url{https://bioconductor.org/config.yaml}
+#' for full offline use and version validation.
 #'
 #' @param pkgs `character()` vector of package names to install or
 #'     update.  A missing value updates installed packages according
@@ -278,6 +325,8 @@
 #' @param checkBuilt `logical(1)`. If `TRUE` a package built under an
 #'     earlier major.minor version of R (e.g., 3.4) is considered to
 #'     be old.
+#' @param force `logical(1)`. If `TRUE` re-download a package that is
+#'     currently up-to-date.
 #' @param version `character(1)` _Bioconductor_ version to install,
 #'     e.g., `version = "3.8"`. The special symbol `version = "devel"`
 #'     installs the current 'development' version.
@@ -321,7 +370,7 @@
 #' @export
 install <-
     function(pkgs = character(), ..., site_repository = character(),
-        update = TRUE, ask = TRUE, checkBuilt = FALSE,
+        update = TRUE, ask = TRUE, checkBuilt = FALSE, force = FALSE,
         version = BiocManager::version())
 {
     stopifnot(
@@ -345,12 +394,13 @@ install <-
     action <- if (cmp < 0) "Downgrade" else "Upgrade"
     repos <- .repositories(site_repository, version = version)
 
+    vout <- .valid_out_of_date_pkgs(pkgs = inst,
+        repos = repos, ..., checkBuilt = checkBuilt,
+        site_repository = site_repository)
+
     if (cmp != 0L) {
         pkgs <- unique(c("BiocVersion", pkgs))
-        valist <- .valid(
-            site_repository = site_repository, version = version,
-            checkBuilt = checkBuilt
-        )
+        valist <- .valid_result(vout, pkgs = inst)
         npkgs <- .install_n_invalid_pkgs(valist) + length(pkgs)
         if (!length(pkgs)-1L) {
             .install_ask_up_or_down_grade(version, npkgs, cmp, ask) ||
@@ -369,17 +419,19 @@ install <-
         }
     }
 
-    .message(
-        "Bioconductor version %s (BiocManager %s), %s",
-        version, packageVersion("BiocManager"),
-        sub(" version", "", R.version.string)
-    )
+    .message(.version_string(version))
 
-    pkgs <- .install(pkgs, repos = repos, ...)
+    pkgs <- .install(
+        pkgs, vout[["out_of_date"]], instPkgs = inst, repos = repos,
+        checkBuilt = checkBuilt, force = force, ...
+    )
     if (update && cmp == 0L) {
         .install_update(repos, ask, checkBuilt = checkBuilt, ...)
     } else if (cmp != 0L) {
-        .install_updated_version(valist, update, repos, ...)
+        .install_updated_version(
+            valist, update, vout[["out_of_date"]], inst, repos, force = force,
+            ...
+        )
     }
 
     invisible(pkgs)
